/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

  Lecture du clavier numérique de la serrure de porte FACTO
  Code Source pour Arduino UNO v3, compilé sur Arduino IDE
  Électro-Bidouilleur, Avril 2021. http://bidouilleur.ca

  Licence de droits: Attribution 4.0 International (CC BY 4.0)  
  Lisez https://creativecommons.org/licenses/by/4.0/deed.fr
*/

#define Broche_Data 8                                               // Assignation de la broche slave select de SPI

// Fonction D'initialisation, style Arduino
void setup()
{
  Serial.begin(9600) ;                                              // Configurer le port série
  pinMode(Broche_Data, INPUT);                                      // Configurer la broche de donnée en entrée
}


// Fonction de boucle principale, style Arduino
void loop()
{
  unsigned long previousMillis;                                     // La variable qui conserve la valeur de millis() pour fins de calcul de délai.
  unsigned int serie_bits;                                          // La variable qui conserve la trame (série de bits)
  char bitnum;                                                      // Compteur de boucle.

  // Lecture d'une trame de bits complète
  if (digitalRead(Broche_Data))                                     // Signal haut présent? 
  {                                                                 // Oui, donc début de la réception de la trame.
    previousMillis = millis();                                      // sauvegarder la valeur de millis() de départ
    while (digitalRead(Broche_Data));                               // Attendre que l'impulsion de debut redescende
    if ((millis() - previousMillis) > 15);                          // Est-ce que l'impulsion a duré au moins 15ms?
    {                                                               // Oui, alors c'est une impulsion de départ
      serie_bits = 0;                                               // Effacer la variable des bits reçus.

      // Recevoir les 9 bits de donnée suivants
      for (bitnum = 0; bitnum < 9; bitnum++)                        // Boucler pour les 9 bits à recevoir
      {   
        while (!digitalRead(Broche_Data));                          // Attendre pour le niveau haut de l'impulsion de bit
        delayMicroseconds(500);                                     // Attendre 500 microsecondes avant de lire le bit
        serie_bits = (serie_bits<<1) + digitalRead(Broche_Data);    // lire et stocker le bit
        while (digitalRead(Broche_Data));                           // Attendre que le bit redescende
      }

      // Afficher la série de bits reçue (les zéros qui précèdent seront aussi affichés)
      Serial.print("Reçu: ") ;
      for (bitnum = 8; bitnum >= 0; bitnum--)                       // 9 bits à afficher le MSB en premier
      {
        if ((serie_bits>>bitnum) & 1) Serial.print("1");            // Isoler et lire le bit. Afficher "1" si le bit =1,
        else Serial.print("0");                                     // sinon afficher "0"
      }

      // Décoder et afficher le bouton pressé
      Serial.print("  Décodé: ") ;
      if ((serie_bits & 0b111100000) == 0b110100000)       Serial.println("Délai Expiré") ;                       // Message "Delai expire" reçu? Afficher un message.
      else if ((serie_bits & 0b111100000) == 0b000000000)  Serial.println((serie_bits & 0b000011110)>>1,DEC) ;    // sinon, message "bouton appuyé" reçu? Masquer et afficher le code du bouton.
    }
  }
}
